import { LogLevel, MockOptions, MockServerPluginOptions, ServerBuildOption } from "./types-DPzh7nJq.js";
import EventEmitter from "node:events";
import { FSWatcher } from "node:fs";
import { Compiler, RspackOptionsNormalized, RspackPluginInstance } from "@rspack/core";
import { Server } from "node:http";
import { Http2SecureServer } from "node:http2";

//#region src/core/logger.d.ts
interface Logger {
  debug: (msg: string, level?: boolean | LogLevel) => void;
  info: (msg: string, level?: boolean | LogLevel) => void;
  warn: (msg: string, level?: boolean | LogLevel) => void;
  error: (msg: string, level?: boolean | LogLevel) => void;
}
declare const logLevels: Record<LogLevel, number>;
declare function createLogger(prefix: string, defaultLevel?: LogLevel): Logger;
//#endregion
//#region src/core/mockCompiler.d.ts
interface MockCompilerOptions {
  alias?: Record<string, false | string | (string | false)[]>;
  plugins: RspackPluginInstance[];
  cwd?: string;
  include: string | string[];
  exclude: string | string[];
  logger: Logger;
}
declare function createMockCompiler(options: MockCompilerOptions): MockCompiler;
declare class MockCompiler extends EventEmitter {
  options: MockCompilerOptions;
  cwd: string;
  mockWatcher: FSWatcher;
  moduleType: "cjs" | "esm";
  entryFile: string;
  private _mockData;
  private fileFilter;
  private watchInfo?;
  compiler?: Compiler | null;
  constructor(options: MockCompilerOptions);
  get mockData(): Record<string, MockOptions>;
  run(): Promise<void>;
  close(): void;
  updateAlias(alias: Record<string, false | string | (string | false)[]>): void;
  updateMockEntry(): Promise<void>;
  getMockFiles(): Promise<string[]>;
  watchMockFiles(): void;
}
//#endregion
//#region src/core/resolvePluginOptions.d.ts
interface ResolvedCompilerOptions {
  alias: Record<string, false | string | (string | false)[]>;
  proxies: (string | ((pathname: string, req: any) => boolean))[];
  wsProxies: (string | ((pathname: string, req: any) => boolean))[];
  plugins: RspackPluginInstance[];
  context?: string;
}
type ResolvePluginOptions = Required<Omit<MockServerPluginOptions, "build">> & ResolvedCompilerOptions & {
  logger: Logger;
  build: false | ServerBuildOption;
};
//#endregion
//#region src/core/mockMiddleware.d.ts
interface MiddlewareOptions {
  alias: Record<string, false | string | (string | false)[]>;
  proxies: (string | ((pathname: string, req: any) => boolean))[];
  context?: string;
  plugins: RspackPluginInstance[];
}
declare function createMockMiddleware(compiler: MockCompiler, options: ResolvePluginOptions): (middlewares: Middleware[], reload?: () => void) => Middleware[];
type SetupMiddlewaresFn = NonNullable<NonNullable<RspackOptionsNormalized["devServer"]>["setupMiddlewares"]>;
type Middleware = SetupMiddlewaresFn extends ((middlewares: (infer T)[], devServer: any) => void) ? T : never;
type Server$1 = SetupMiddlewaresFn extends ((middlewares: any, devServer: infer T) => void) ? T : never;
//#endregion
//#region src/core/mockWebsocket.d.ts
interface MockSocketOptions {
  wsProxies: (string | ((pathname: string, req: any) => boolean))[];
  cookiesOptions: MockServerPluginOptions["cookiesOptions"];
  logger: Logger;
}
declare function mockWebSocket(compiler: MockCompiler, httpServer: Server | Http2SecureServer, {
  wsProxies: proxies,
  cookiesOptions,
  logger
}: MockSocketOptions): void;
//#endregion
export { Logger, Middleware, MiddlewareOptions, MockCompiler, MockCompilerOptions, MockSocketOptions, ResolvePluginOptions, Server$1 as Server, createLogger, createMockCompiler, createMockMiddleware, logLevels, mockWebSocket };