const require_chunk = require('./chunk-CUT6urMc.cjs');
const require_logger = require('./logger-C0V8Cvvd.cjs');
const require_resolvePluginOptions = require('./resolvePluginOptions-Da5uqlBx.cjs');
const __pengzhanbo_utils = require_chunk.__toESM(require("@pengzhanbo/utils"));
const node_path = require_chunk.__toESM(require("node:path"));
const node_process = require_chunk.__toESM(require("node:process"));
const picocolors = require_chunk.__toESM(require("picocolors"));
const __rspack_core = require_chunk.__toESM(require("@rspack/core"));
const node_http = require_chunk.__toESM(require("node:http"));
const portfinder = require_chunk.__toESM(require("portfinder"));

//#region src/rsbuild.ts
function pluginMockServer(options = {}) {
	return {
		name: "plugin-mock-server",
		setup(api) {
			const rsbuildConfig = api.getRsbuildConfig();
			const resolvedOptions = require_resolvePluginOptions.resolvePluginOptions(options, {
				proxies: resolveConfigProxies(rsbuildConfig),
				alias: {},
				context: api.context.rootPath,
				plugins: [new __rspack_core.default.DefinePlugin(rsbuildConfig.source?.define || {})]
			});
			if (node_process.default.env.NODE_ENV === "production") {
				if (resolvedOptions.build) api.onAfterBuild(async () => {
					const config = api.getNormalizedConfig();
					await require_resolvePluginOptions.buildMockServer(resolvedOptions, node_path.default.resolve(node_process.default.cwd(), config.output.distPath.root || "dist"));
				});
				return;
			}
			const mockCompiler = require_resolvePluginOptions.createMockCompiler(resolvedOptions);
			api.modifyRsbuildConfig((config) => {
				updateServerProxyConfigByHttpMock(config);
				const mockMiddleware = require_resolvePluginOptions.createMockMiddleware(mockCompiler, resolvedOptions);
				config.dev ??= {};
				config.dev.setupMiddlewares ??= [];
				config.dev.setupMiddlewares.push((middlewares, server$1) => {
					mockMiddleware(middlewares, () => server$1.sockWrite("static-changed"));
				});
			});
			let port = 3079;
			const shouldMockWs = (0, __pengzhanbo_utils.toArray)(resolvedOptions.wsPrefix).length > 0;
			if (shouldMockWs) api.modifyRsbuildConfig(async (config) => {
				const defaultPort = (config.server?.port || port) + 1;
				port = await (0, portfinder.getPortPromise)({ port: defaultPort });
				updateServerProxyConfigByWSMock(config, options.wsPrefix || [], port);
			});
			let server;
			function startMockServer() {
				mockCompiler.run();
				if (shouldMockWs) {
					server = (0, node_http.createServer)();
					require_logger.mockWebSocket(mockCompiler, server, resolvedOptions);
					server.listen(port);
				}
			}
			function close() {
				mockCompiler.close();
				server?.close();
			}
			api.onAfterCreateCompiler(({ compiler }) => {
				if ("compilers" in compiler) compiler.compilers.forEach((compiler$1) => {
					mockCompiler.updateAlias(compiler$1.options.resolve?.alias || {});
				});
				else mockCompiler.updateAlias(compiler.options.resolve?.alias || {});
			});
			api.onAfterStartDevServer(startMockServer);
			api.onAfterStartProdServer(startMockServer);
			api.onExit(close);
		}
	};
}
function onProxyError(err, _req, res) {
	console.error(picocolors.default.red(err?.stack || err.message));
	res.statusCode = 500;
	res.end();
}
function updateServerProxyConfigByHttpMock(config) {
	if (!config.server?.proxy) return;
	if ((0, __pengzhanbo_utils.isArray)(config.server.proxy)) config.server.proxy = config.server.proxy.map((item) => {
		if (typeof item !== "function" && !item.ws) {
			const onProxyReq = item.onProxyReq;
			const onError = item.onError;
			return {
				...item,
				onError: onError || onProxyError,
				onProxyReq: (proxyReq, req, ...args) => {
					onProxyReq?.(proxyReq, req, ...args);
					require_logger.rewriteRequest(proxyReq, req);
				}
			};
		}
		return item;
	});
	else if ("target" in config.server.proxy) {
		const onProxyReq = config.server.proxy.onProxyReq;
		config.server.proxy.onProxyReq = (proxyReq, req, ...args) => {
			onProxyReq?.(proxyReq, req, ...args);
			require_logger.rewriteRequest(proxyReq, req);
		};
		config.server.proxy.onError ??= onProxyError;
	} else if (config.server.proxy) {
		const proxy = config.server.proxy;
		Object.keys(proxy).forEach((key) => {
			const target = proxy[key];
			const options = typeof target === "string" ? { target } : target;
			if (options.ws) return;
			const { onProxyReq, onError,...rest } = options;
			proxy[key] = {
				...rest,
				onProxyReq: (proxyReq, req, ...args) => {
					onProxyReq?.(proxyReq, req, ...args);
					require_logger.rewriteRequest(proxyReq, req);
				},
				onError: onError || onProxyError
			};
		});
	}
}
function updateServerProxyConfigByWSMock(config, wsPrefix, port) {
	config.server ??= {};
	const proxy = config.server.proxy ??= {};
	const wsTarget = `ws://localhost:${port}`;
	const prefix = (0, __pengzhanbo_utils.toArray)(wsPrefix);
	const has = (context) => typeof context === "string" && prefix.includes(context);
	const used = /* @__PURE__ */ new Set();
	function updateProxy(item) {
		if ((0, __pengzhanbo_utils.isArray)(item.context)) item.context = item.context.filter(has);
		else if (has(item.context)) {
			used.add(item.context);
			item.target = wsTarget;
		}
	}
	if ((0, __pengzhanbo_utils.isArray)(proxy)) {
		for (const item of proxy) if (typeof item !== "function" && item.context && item.ws) updateProxy(item);
		prefix.filter((context) => !used.has(context)).forEach((context) => proxy.push({
			context,
			target: wsTarget
		}));
	} else if ("target" in proxy) {
		if (proxy.ws) {
			updateProxy(proxy);
			const list = config.server.proxy = [proxy];
			prefix.filter((context) => !used.has(context)).forEach((context) => list.push({
				context,
				target: wsTarget
			}));
		}
	} else {
		Object.entries(proxy).forEach(([, opt]) => {
			if (typeof opt !== "string" && opt.ws) updateProxy(opt);
		});
		prefix.filter((context) => !used.has(context)).forEach((context) => {
			proxy[context] = {
				target: wsTarget,
				ws: true
			};
		});
	}
}
function resolveConfigProxies(config) {
	config.server ??= {};
	const proxy = config.server.proxy ??= {};
	const proxies = [];
	if ((0, __pengzhanbo_utils.isArray)(proxy)) {
		for (const item of proxy) if (typeof item !== "function" && item.context && !item.ws) proxies.push(...(0, __pengzhanbo_utils.toArray)(item.context));
	} else if ("target" in proxy) {
		if (!proxy.ws) proxies.push(...(0, __pengzhanbo_utils.toArray)(proxy.context));
	} else Object.entries(proxy).forEach(([context, opt]) => {
		if (typeof opt === "string" || !opt.ws) proxies.push(context);
	});
	return proxies;
}

//#endregion
exports.pluginMockServer = pluginMockServer;